# Duo, A crazy eights game.
# Copyright (C) 2009 The Duo developers.
#
# This file is part of Duo.
#
# Duo is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
The dumber robot written for Duo ever.

Dumb is the dumber possible player -- while always playing valid
moves. It will always choose an action randomly among all its possible
moves.
"""
from __future__ import with_statement

import logging
import threading
import time

from duo import ai
from duo import game
from duo import protocol
from duo import referee

NAMES = [ 'Zaahir', 'Zacharias', 'Zala', 'Zebulon', 'Zed', 'Zella',
          'Zeus', 'Ziva', 'Zlatan', 'Zoe', 'Zuri', 'Zvonimira' ]

logger = logging.getLogger('duo.ai.dumb')


class AI(ai.AbstractAI):
    def __init__(self):
        ai.AbstractAI.__init__(self)
        logger.debug('AI.__init__')
        self._condition = threading.Condition()
        self._end = False

    def stop_game(self):
        with self._condition:
            self._end = True
            self._condition.notify()

    def on_winner(self, payload, emitter, args):
        with self._condition:
            logger.debug(
                '%s congratulates %s...' % (self.player.name, payload))
            self._end = True
            self._condition.notify()

    def run(self):
        """TODO"""
        logger.debug('%s is back at the table.' % self.player.name)
        self._condition.acquire()
        while not self._end:
            # Wait for our turn.
            self._condition.wait()

            # Issue a move.
            if not self._end and len(self.player.actions):
                logger.debug('%s is thinking...' % self.player.name)
                time.sleep(1)
                self.player.send_to_referee(
                    protocol.ActionMessage(self.player.actions.pop()))
        logger.debug('%s asks for a drink.' % self.player.name)
        self._condition.release()

        # Reset between rounds.
        self._end = False
