# Duo, A crazy eights game.
# Copyright (C) 2009 The Duo developers.
#
# This file is part of Duo.
#
# Duo is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""The transport layer generic classes unit tests.
"""

from duo.transport import Context
from duo.transport import Endpoint
from duo.transport import ProtocolPacket
from duo.transport import TransportJoinPacket
from duo.transport import TransportEscapePacket

import unittest


class EndpointMockup(object):
    def __init__(self, id):
        self._id = id
        self._context = None

    def get_id(self):
        return self._id

    def set_context(self, context):
        if self._context:
            self._context.remove_endpoint_without_check(self)
        self.set_context_without_check(context)

    def set_context_without_check(self, context):
        self._context = context

    def get_context(self):
        return self._context


class TestProtocolPacket(unittest.TestCase):
    def test_get_payload(self):
        packet = ProtocolPacket('payload')
        self.assertEqual(packet.payload, 'payload')


class TestTransportJoinPacket(unittest.TestCase):
    def test_get_type(self):
        packet = TransportJoinPacket('payload')
        self.assertEqual(packet.payload, 'payload')


class TestTransportEscapePacket(unittest.TestCase):
    def test_get_type(self):
        packet = TransportEscapePacket('payload')
        self.assertEqual(packet.payload, 'payload')


class testContext(unittest.TestCase):
    def test_add_and_remove_endpoint(self):
        context = Context('c')
        endpoint = EndpointMockup('endpoint')
        context.add_endpoint(endpoint)
        self.assert_(endpoint in context.get_endpoints())
        self.assert_(endpoint.get_context() is context)

        context.remove_endpoint(endpoint)
        self.assert_(not endpoint in context.get_endpoints())
        self.assert_(endpoint.get_context() is None)

    def test_add_and_remove_endpoint_without_check(self):
        context = Context('c')
        endpoint = EndpointMockup('endpoint')
        context.add_endpoint_without_check(endpoint)
        self.assert_(endpoint in context.get_endpoints())
        self.assert_(endpoint.get_context() is None)

        endpoint.set_context(context)
        context.remove_endpoint_without_check(endpoint)
        self.assert_(not endpoint in context.get_endpoints())
        self.assert_(endpoint.get_context() is context)

    def test_get_parent(self):
        root = Context('root')
        child = Context('child', root)
        self.assert_(child.get_parent() is root)
        self.assert_(root.get_parent() is None)

    def test_get_name(self):
        context = Context('root')
        self.assert_(context.get_name() == 'root')

    def test_get_or_create_context(self):
        root = Context('root')
        self.assert_(root.get_or_create_context('root') is root)

        child = root.get_or_create_context('child')
        self.assert_(root.get_or_create_context('child') is child)
        self.assert_(child.get_or_create_context('child') is child)

    def test_get_child_contexts(self):
        root = Context('root')
        child = root.get_or_create_context('child1')
        self.assert_(child.get_parent() is root)
        self.assert_(child in root.get_child_contexts())


class testEndpoint(unittest.TestCase):

     #     def __init__(self, aID, aCallbacks):
     #         self._context = None
     #         self._id = aID
     #         self._cb = aCallbacks

     def test_get_id(self):
         endpoint = Endpoint('e1', {})
         self.assertEqual(endpoint.get_id(), 'e1')


#     def setContext(self, aContext):
#         if self._context:
#             self._context.removeEndpointWithoutCheck(self)
#         self.setContextWithoutCheck(aContext)

#     def setContextWithoutCheck(self, aContext):
#         logging.getLogger('duo.transport').debug(
#             'Attaching %s to %s' % (self, aContext))
#         self._context = aContext

#     def getContext(self):
#         return self._context

#     def notify(self, aByteArray, aFrom):
#         """Notify the reception of a message."""
#         lPacket = cPickle.loads(aByteArray)
#         try:
#             lCallback = self._cb[lPacket.getType()]
#             logging.getLogger('duo.transport').debug('%s notifies %s message in %s ' %
#                     (self.getID(), lPacket.getType(), lCallback.__name__))
#             lCallback.__call__(lPacket.getPayload(), aFrom)
#         except MemoryError, e:
#             logging.getLogger('duo.transport').error(
#                 'Endpoint.notify %s: %s' % (lPacket.getPayload(), e))

#     def sendTo(self, aPayload, aTo):
#         """Send a message to another endpoint."""
#         self._doSend(cPickle.dumps(Packet(Packet.TYPE_PROTOCOL, aPayload)), aTo)

#     def broadcast(self, aPayload):
#         """Send a message to every connected endpoints in the current context.
#         """
#         self._doSend(cPickle.dumps(Packet(Packet.TYPE_PROTOCOL, aPayload)), None)

#     def getContexts(self):
#         """Return full information on existing contexts."""
#         raise NotImplementedError, \
#               '%s shall define notify' % self.__class__.__name__

#     def joinContext(self, aContextName):
#         """Attach this endpoint to the given context."""
#         raise NotImplementedError, \
#               '%s shall define notify' % self.__class__.__name__

#     def escape(self):
#         """Detach from the current context.

#         Once detached, the endpoint is automatically attached to the
#         parent context. If there is no parent, the endpoint is disconnected.
#         """
#         raise NotImplementedError, \
#               '%s shall define notify' % self.__class__.__name__

#     def _doSend(self, aPacket, aTo):
#         """Send a packet.

#         If aTo is None, the packet has to be broadcasted.
#         """
#         raise NotImplementedError, \
#               '%s shall define notify' % self.__class__.__name__
