# Duo, A crazy eights game.
# Copyright (C) 2009 The Duo developers.
#
# This file is part of Duo.
#
# Duo is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import logging, math, mutex, os, sys

from duo import protocols, referee, ai

try:
    import pygtk
    pygtk.require('2.0')
except Exception, e:
    logging.getLogger('duo.ui').error('gnomish: No PyGTK 2.0: %s', e)
    sys.exit(1)

try:
    import gtk, gtk.glade
    import gobject
except Exception, e:
    logging.getLogger('duo.ui').error('gnomish: No GTK: %s', e)
    sys.exit(1)

try:
    import PIL.Image
except Exception, e:
    logging.getLogger('duo.ui').error('gnomish: No PIL: %s', e)
    sys.exit(1)


gobject.threads_init()

# TODO
GLADE_FILE = '/home/david/src/duo/glade/duo.glade'
CARDSET='classic'


class NewGameDialog(object):
    DIALOG = 'new_game_dialog'
    NAME_ENTRY = 'name_entry'
    AI_ENTRY = 'ai_entry'
    PROTOCOL_COMBO = 'protocol_combo'
    OPTIONS_NOTEBOOK = 'options_notebook'
    OK_BUTTON = 'okbutton'
    CANCEL_BUTTON = 'cancelbutton'

    def __init__(self):
        # Load the Glade design and build the dialog.
        self.__design = gtk.glade.XML(GLADE_FILE, self.DIALOG)
        self.__dialog = self.__design.get_widget(self.DIALOG)
        self.__combo = self.__design.get_widget(self.PROTOCOL_COMBO)
        self.__book = self.__design.get_widget(self.OPTIONS_NOTEBOOK)

        # Plug callbacks.
        self.__design.signal_autoconnect(
            {
            'on_'+ self.PROTOCOL_COMBO + '_changed': (self.__onNewProtocol),
            'on_' + self.OK_BUTTON + '_clicked': (self.__onOk),
            'on_' + self.CANCEL_BUTTON + '_clicked': (self.__onCancel)
            }
        )

        # The protocol options.
        self.__options = {}

        # Complete the dialog with live information.
        self.__initialize()



    def run(self):
        return self.__dialog.run()


    def getGameParams(self):
        return (self.getPlayerName(),
                self.getNbAIPlayers(),
                self.getProtocol(),
                self.getProtocolOptions())


    def getPlayerName(self):
        return self.__design.get_widget(self.NAME_ENTRY).get_text()


    def getNbAIPlayers(self):
        return self.__design.get_widget(self.AI_ENTRY).get_value()


    def getProtocol(self):
        return self.__combo.get_active_text().lower()


    def getProtocolOptions(self):
        return self.__options[self.__combo.get_active_text().lower()]


    def __initialize(self):
        # Complete the protocols list.
        self.__combo.set_model(gtk.ListStore(gobject.TYPE_STRING))
        lCell = gtk.CellRendererText()
        self.__combo.pack_start(lCell, True)
        self.__combo.add_attribute(lCell, 'text', 0)
        for lProtocol in protocols.getProtocols():
            self.__addProtocol(lProtocol)


    def __addProtocol(self, aProtocol):
        self.__options[aProtocol] = protocols.getProtocolOptions(aProtocol)
        lPane = gtk.Table(len(self.__options[aProtocol]), 2)
        lIdx = 0
        for lOption in self.__options[aProtocol]:
            lWidget = None
            if type(lOption.getValue()) is str:
                lWidget = gtk.Label(lOption.getPrompt())
                lWidget.show()
                lPane.attach(lWidget, 0, 1, lIdx, lIdx + 1)
                lWidget = gtk.Entry()
                lWidget.set_text(lOption.getValue())
                lWidget.connect(
                    'changed', lambda w, o: o.setValue(w.get_text()), lOption)
                lWidget.show()
                lPane.attach(
                    lWidget, 1, 2, lIdx, lIdx + 1, xpadding = 5, ypadding = 5)
            elif type(lOption.getValue()) is int:
                lWidget = gtk.Label(lOption.getPrompt())
                lWidget.show()
                lPane.attach(lWidget, 0, 1, lIdx, lIdx + 1)
                lWidget = gtk.SpinButton(gtk.Adjustment(1, 1, 9999, 1))
                lWidget.set_value(lOption.getValue())
                lWidget.connect(
                    'value-changed', lambda w, o: o.setValue(w.get_value_as_int()), lOption)
                lWidget.show()
                lPane.attach(
                    lWidget, 1, 2, lIdx, lIdx + 1, xpadding = 5, ypadding = 5)
            elif type(lOption.getValue()) is bool:
                lWidget = gtk.CheckButton(lOption.getPrompt())
                lWidget.set_active(lOption.getValue())
                lWidget.connect(
                    'toggled', lambda w, o: o.setValue(w.get_active()), lOption)
                lWidget.show()
                lPane.attach(
                    lWidget, 1, 3, lIdx, lIdx + 1, xpadding = 5, ypadding = 5)
            lPane.show()
            lIdx = lIdx + 1
        self.__combo.append_text(aProtocol.capitalize())
        self.__book.append_page(lPane, gtk.Label(aProtocol.capitalize()))


    def __onNewProtocol(self, aCombo):
        for lIdx in range(self.__book.get_n_pages()):
            lLabel = self.__book.get_tab_label(self.__book.get_nth_page(lIdx))
            if lLabel.get_text() == self.__combo.get_active_text():
                self.__book.set_current_page(lIdx)
                return


    def __onOk(self, aButton):
        self.__dialog.hide()


    def __onCancel(self, aButton):
        self.__dialog.hide()



class Card(object):
    """Card class"""
    def __init__(self, aBoard, aRawCard, aIsVertical = True, aX = 0, aY = 0):
        self.__x = aX
        self.__y = aY
        self.__type = aRawCard.getType()
        self.__color = aRawCard.getColor()
        self.__isVertical = aIsVertical
        self.__isVisible = False

        # Card images
        self.__front = None
        self.__back = None

        self.__initialize(aBoard)


    def __str__(self):
        return self.__type + self.__color


    def __repr__(self):
        return self.__str__()


    def __initialize(self, aBoard):
        # If no coordinates, put the card on the stack.
        if self.__x == 0: self.__x = aBoard.width / 2 + aBoard.cardSpace
        if self.__y == 0: self.__y = aBoard.height / 2 - aBoard.cardHeight / 2

        try:
            # TODO: move path in config
            self.__back = PIL.Image.open(
                os.path.join('.', CARDSET, 'back.png')).convert('RGB')
            self.__back = self.__back.resize(
                (aBoard.cardWidth, aBoard.cardHeight), PIL.Image.ANTIALIAS)

            self.__front = PIL.Image.open(self.getCardFile()).convert('RGB')
            self.__front = self.__front.resize(
                (aBoard.cardWidth, aBoard.cardHeight), PIL.Image.ANTIALIAS)

            if not self.__isVertical:
                self.rotate(90)

        except Exception, e:
            logging.getLogger('duo.ui').error(
                'gnomish.Card: You didn\'t install correctly the card set: %s',
                e)


    def setCoordinates(self, aX, aY):
        self.__x = aX
        self.__y = aY


    def setVisible(self, aValue = True):
        self.__isVisible = aValue


    def rotate(self, aAngle):
        self.__front = self.__front.rotate(aAngle)
        self.__back = self.__back.rotate(aAngle)


    def getColor(self):
        return self.__color


    def getType(self):
        return self.__type


    def getRawCard(self):
        return referee.Card(self.__type, self.__color)


    def getCoordinates(self):
        return self.__x, self.__y


    def getDimensions(self):
        return self.__front.size


    def getData(self):
        if self.__isVisible:
            return self.__front.tostring()
        else:
            return self.__back.tostring()


    def getCardFile(self):
        lNameRoot = ''
        if self.__type == 'D':
            lNameRoot = '+2-' + referee.Card.COLORS[self.__color].lower()
        elif self.__type == 'R':
            lNameRoot = 'inv-' + referee.Card.COLORS[self.__color].lower()
        elif self.__type == 'P':
            lNameRoot = 'pas-' + referee.Card.COLORS[self.__color].lower()
        elif self.__type == 'W':
            lNameRoot = 'jok'
        elif self.__type == 'X':
            lNameRoot = '+4'
        else:
            lNameRoot = \
                self.__type + '-' + referee.Card.COLORS[self.__color].lower()

        # TODO: move path in config
        return os.path.join('.', CARDSET, lNameRoot + ".png")



class Board(object):
    BOARD = 'drawing_area'
    SOUTH, WEST, NORTH, EAST, DECK = range(5)

    def __init__(self, aApp, aDesign):
        self.__app = aApp
        self.__view = aDesign.get_widget(self.BOARD)
        self.__gc = self.__view.get_style().fg_gc[gtk.STATE_NORMAL]

        aDesign.signal_autoconnect(self)

        # For the animations.
        self.__mutex = mutex.mutex()

        # Create a new backing pixmap, and a background for animations.
        lX, lY, lWidth, lHeight = self.__view.get_allocation()
        self.__width = lWidth
        self.__height= lHeight
        self.__pixmap = gtk.gdk.Pixmap(self.__view.window, lWidth, lHeight)
        self.__background = gtk.gdk.Pixmap(self.__view.window, lWidth, lHeight)
        self.__initialize()

        # The heap card
        self.__heap = None


    def __initialize(self):
        self.__view.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse('dark green'))

        self.__pixmap.draw_rectangle(
            self.__view.get_style().bg_gc[gtk.STATE_NORMAL], True, 0, 0, self.__width, self.__height)

        # Draw any card for the deck, since we are only interested in
        # its back.
        lCard = Card(self, referee.Card('X'), Board.DECK)
        lX, lY = lCard.getCoordinates()
        lWidth, lHeight = lCard.getDimensions()
        lData = lCard.getData()
        self.__pixmap.draw_rgb_image(
            self.__gc, lX, lY, lWidth, lHeight, gtk.gdk.RGB_DITHER_NONE, lData)


    width = property(lambda self: self.__width)
    height = property(lambda self: self.__height)
    cardWidth = property(lambda self: (self.__height / 5) * 2/3)
    cardHeight = property(lambda self: self.__height / 5)
    cardSpace = property(lambda self: (self.__height / 5 ) / 3)
    cardXBorder = property(lambda self: ((self.__height / 5) * 2/3 ) / 3)
    cardYBorder = property(lambda self: ((self.__height / 5) * 2/3 ) / 3)


    def createCard(self, aRawCard, aIsVertical = True):
        lCard = Card(self, aRawCard, aIsVertical)
        return lCard


    def setHeapCard(self, aRawCard):
        """Move a card from the deck to the heap.
        """
        logging.getLogger('duo.ui').debug(
            'gnomish.Board.setHeapCard %s', aRawCard)
        self.__heap = self.createCard(aRawCard, Board.DECK)
        self.__heap.setVisible()
        lX = self.width / 2 - self.cardSpace - self.cardWidth
        lY = self.height / 2 - self.cardHeight / 2
        self.__mutex.lock(self.__startCardAnimation, (self.__heap, (lX, lY)))
        return False


    def dealCard(self, aCard, aPlayer, aRank):
        """Move a card from the deck to the hand of a player.
        """
        logging.getLogger('duo.ui').debug(
            'gnomish.Board.dealCard %s', (aCard, aPlayer, aRank))

        # Compute the target coordinates
        lPosition = aPlayer.getPosition()
        lWidth, lHeight = aCard.getDimensions()
        lX, lY = 0, 0
        if lPosition == Board.SOUTH:
            lX = aRank * self.cardSpace + self.cardXBorder
            lY = self.height - lHeight - self.cardYBorder

        elif lPosition == Board.WEST:
            lX = self.cardYBorder
            lY = self.cardXBorder + aRank * self.cardSpace

        elif lPosition == Board.NORTH:
            lX = self.width - self.cardHeight - \
                 aRank * self.cardSpace - self.cardXBorder
            lY = self.cardYBorder

        elif lPosition == Board.EAST:
            lX = self.width - self.cardHeight - self.cardYBorder
            lY = self.height - self.cardWidth - \
                 self.cardXBorder - aRank * self.cardSpace

        self.__mutex.lock(self.__startCardAnimation, (aCard, (lX, lY)))
        return False

    def raiseCard(self):
        # TODO
        return False


    def playCard(self, aCard, aPlayer):
        """Move a card from the hand of a player to the heap.
        """
        logging.getLogger('duo.ui').debug(
            'gnomish.Board.playCard %s', (aCard, aPlayer))

        #
        # TODO: should queue this first drawing with mutex...
        #

        # Remove the card from the hand to create animation background.
        lPosition = aPlayer.getPosition()
        lCards = aPlayer.getCards()

        if lPosition == Board.SOUTH:
            self.__pixmap.draw_rectangle(
                self.__gc, True,
                self.cardXBorder,
                self.height - self.cardHeight - self.cardYBorder,
                self.cardSpace, self.cardHeight)
#                len(lCards) * self.cardSpace + self.cardXBorder, self.cardHeight)

        elif lPosition == Board.WEST:
            self.__pixmap.draw_rectangle(
                self.__gc, True,
                self.cardXBorder, self.cardYBorder,
                self.cardSpace, self.cardHeight)
            aCard.rotate(-90)
            aCard.setVisible()

        # OTHERS TODO
        else:
            logging.getLogger('duo.ui').warning(
                'gnomish.Board.playCard TO COMPLETE!')

        for lCard in lCards:
            if lCard != aCard:
                self.__mutex.lock(self.__drawCard, lCard)


        # Target coordinates are those of the heap
        lX = self.width / 2 - self.cardSpace - self.cardWidth
        lY = self.height / 2 - self.cardHeight / 2

        self.__mutex.lock(self.__startCardAnimation, (aCard, (lX, lY)))
        return False

    def packCards(self, aPlayer):
        logging.getLogger('duo.ui').debug(
            'gnomish.Board.packCards %s', aPlayer)

        # Redraw the remaining cards.
        lPosition = aPlayer.getPosition()
        lCards = aPlayer.getCards()

        for lCard in lCards:
            self.__drawCard(lCard, aRefresh = False)

        # Clean the zone after the last card.
        #  TODO

        # TODO: refine the refresh zone
        self.__view.queue_draw_area(0, 0, self.width, self.height)
        return False


    def revertCard(self, aCard, aPlayer, aRank):
        logging.getLogger('duo.ui').debug(
            'gnomish.Board.revertCard %s', (aCard, aPlayer, aRank))

        self.__mutex.lock(self.__drawCard, self.__heap)

        # Compute the target coordinates
        lPosition = aPlayer.getPosition()
        lWidth, lHeight = aCard.getDimensions()
        lX, lY = 0, 0
        if lPosition == Board.SOUTH:
            lX = aRank * self.cardSpace + self.cardXBorder
            lY = self.height - lHeight - self.cardYBorder

        elif lPosition == Board.WEST:
            lX = self.cardYBorder
            lY = self.cardXBorder + aRank * self.cardSpace

        elif lPosition == Board.NORTH:
            lX = self.width - self.cardHeight - \
                 aRank * self.cardSpace - self.cardXBorder
            lY = self.cardYBorder

        elif lPosition == Board.EAST:
            lX = self.width - self.cardHeight - self.cardYBorder
            lY = self.height - self.cardWidth - \
                 self.cardXBorder - aRank * self.cardSpace

        self.__mutex.lock(self.__startCardAnimation, (aCard, (lX, lY)))

        return self.packCards(aPlayer)


    def onExposeEvent(self, aWidget, aEvent):
        """Redraw the screen from the backing pixmap."""
        lX , lY, lWidth, lHeight = aEvent.area
        aWidget.window.draw_drawable(
            self.__gc, self.__pixmap, lX, lY, lX, lY, lWidth, lHeight)
        return False


    def onButtonPressEvent(self, aWidget, aEvent):
        if (aEvent.button != 1):
            return False

        ## TODO: Improve communivation with the player
        lPlayer = self.__app.getPlayers()[0]

        # Remember origin is the upper left corner.
        lDeckBottom = self.height / 2 + self.cardHeight / 2
        lDeckTop = self.height / 2 - self.cardHeight / 2
        lDeckLeft = self.width / 2 + self.cardSpace
        lDeckRight = lDeckLeft + self.cardWidth

        lYBottom = self.height - self.cardYBorder
        lYTop = lYBottom  - self.cardHeight
        lXLeft = self.cardXBorder
        lXRight = lXLeft + (len(lPlayer.getCards())-1) * self.cardSpace \
                  + self.cardWidth
        lCardIdx = min(int((aEvent.x - lXLeft) / self.cardSpace),
                       len(lPlayer.getCards()) - 1)

        if aEvent.y < lDeckBottom and aEvent.y > lDeckTop \
                and aEvent.x > lDeckLeft and aEvent.x < lDeckRight:
            lPlayer.onDrawCb(None)

        elif aEvent.y < lYBottom and aEvent.y > lYTop \
                and aEvent.x > lXLeft and aEvent.x < lXRight:
            lPlayer.play(lPlayer.getCards()[lCardIdx])

        return False


    def onConfigureEvent(self, aWidget, aEvent):
        # TODO resize pixmap
        return False


    def __drawCard(self, aCard, aIsMoving = False, aRefresh = True):
        lX, lY = aCard.getCoordinates()
        lWidth, lHeight = aCard.getDimensions()
        lData = aCard.getData()
        self.__pixmap.draw_rgb_image(
            self.__gc, lX, lY, lWidth, lHeight, gtk.gdk.RGB_DITHER_NONE, lData)
        if aRefresh:
            self.__view.queue_draw_area(lX, lY, lWidth, lHeight)
        if not aIsMoving:
            self.__mutex.unlock()


    def __move(self, aCard, aX, aY):
        lCardX, lCardY = aCard.getCoordinates()
        lWidth, lHeight = aCard.getDimensions()
        lStep = 10
        lAchieved = abs(lCardX - aX) < lStep and abs(lCardY - aY) < lStep

        # Erase the card from the background
        self.__pixmap.draw_drawable(
            self.__gc, self.__background, lCardX, lCardY, lCardX, lCardY,
            lWidth, lHeight)
        self.__view.queue_draw_area(lCardX, lCardY, lWidth, lHeight)

        # Move the card
        if lAchieved:
            aCard.setCoordinates(aX, aY)
        else:
            lAngle = math.atan2(aY - lCardY, aX - lCardX)
            lXDirection = (aX - lCardX) / abs(aX - lCardX + .01)
            lYDirection =  (aY - lCardY) / abs(aY - lCardY + .01)

            lCardX = lStep * math.cos(lAngle) + lCardX
            lCardY = lStep * math.sin(lAngle) + lCardY
            aCard.setCoordinates(int(lCardX), int(lCardY))

        self.__drawCard(aCard, True)

        if lAchieved:
            self.__mutex.unlock()
        return not lAchieved


    def __startCardAnimation(self, aBundle):
        logging.getLogger('duo.ui').debug(
            'gnomish.Board.__startCardAnimation %s', aBundle)
        lCard, (lX, lY) = aBundle

        # Create the background
        self.__background.draw_drawable(
            self.__gc, self.__pixmap, 0, 0, 0, 0, self.width, self.height)

        # Start the animation
        lTimerID = gobject.timeout_add(5, self.__move, lCard, lX, lY)



class Player(object):
    def __init__(
        self, aApp, aName, aProtocol, aOptions, aPosition = Board.SOUTH):
        logging.getLogger('duo.ui').debug(
            'gnomish.Player.__init__ %s', (aName, aProtocol, aOptions))
        self.__app = aApp
        self.__board = aApp.getBoard()
        self.__position = aPosition
        self.__cards = []

        # CATCH errors...
        self.__player = protocols.createPlayer(aName, aProtocol, aOptions)

        # Register to the User Interface.
        self.__readyButton = None
        self.__drawButton = None
        self.__passButton = None
        self.__registerUI()

        # Register protocol events.
        self.__player.register(protocols.Message.DEAL, self.onDealUICb)
        self.__player.register(protocols.Message.HEAP, self.onHeapUICb)
        self.__player.register(protocols.Message.PLAY, self.onPlayUICb)
        self.__player.register(protocols.Message.ERROR, self.onErrorUICb)
        self.__player.register(protocols.Message.TURN, self.onTurnUICb)
        self.__player.register(protocols.Message.PASS, self.onPassUICb)
        self.__player.register(protocols.Message.REVERSE, self.onReverseUICb)

        # Contact the referee.
        self.__player.authenticate()


    def __str__(self):
        return self.__player.getName()


    def __repr__(self):
        return self.__str__()


    def __registerUI(self):
        lBox = gtk.VBox()

        # Put player name.
        lBox.pack_start(gtk.Label(self.getName()))

        # Put buttons if not AI.
        if not self.__player.isRobot():
            self.__readyButton = gtk.Button('Ready')
            self.__readyButton.connect('clicked', self.onReadyCb)
            self.__drawButton = gtk.Button('Draw')
            self.__drawButton.connect('clicked', self.onDrawCb)
            self.__passButton = gtk.Button('Pass')
            self.__passButton.connect('clicked', self.onPassCb)

            lBox.pack_start(self.__readyButton, False)
            lBox.pack_start(self.__drawButton, False)
            lBox.pack_start(self.__passButton, False)

        lBox.show_all()
        self.__app.registerPlayerUI(lBox)


    def getName(self):
        return self.__player.getName()


    def getPosition(self):
        return self.__position


    def getCards(self):
        return self.__cards


    def onReadyCb(self, aWidget):
        """Tells the player is ready to play.

        This method is called when the human player clicks on the Ready button,
        meaning this way he is ready to start playing.
        """
        logging.getLogger('duo.ui').debug('gnomish.Player.onReadyCb')
        self.__player.ready()
        self.__readyButton.set_state(gtk.STATE_INSENSITIVE)


    def onDrawCb(self, aWidget):
        logging.getLogger('duo.ui').debug('gnomish.Player.onDrawCb')
        self.__player.draw()


    def onPassCb(self, aWidget):
        logging.getLogger('duo.ui').debug('gnomish.Player.onPassCb')
        self.__player.passTurn()


    def play(self, aCard):
        logging.getLogger('duo.ui').debug(
            'gnomish.Player.play %s', aCard)
        lColor = aCard.getColor()
        if lColor == 'N':
            lColor = ColorDialog(self.__app).getColor()

        self.__board.playCard(aCard, self)
        self.__player.move(aCard.getRawCard(), lColor)

        if not self.__player.isRobot():
            self.__drawButton.set_state(gtk.STATE_INSENSITIVE)
            self.__passButton.set_state(gtk.STATE_INSENSITIVE)


    def onDealUICb(self, aMessage, aArgs):
        logging.getLogger('duo.ui').debug(
            'gnomish.Player.onDealUICb %s', aMessage.getPayload())

        # First determine if this is the first deal, and the position of
        # the new cards.
        lCards = aMessage.getPayload()
        lFirstIdx = len(self.__player.getHand()) - len(lCards)
        lIdx = lFirstIdx

        # Deal the incoming cards.
        for lRawCard in lCards:
            lCard = self.__board.createCard(lRawCard, self.__position % 2 == 0)
            if not self.__player.isRobot():
                lCard.setVisible()
            self.__cards.append(lCard)
            gobject.idle_add(self.__board.dealCard, lCard, self, lIdx)
            lIdx = lIdx + 1

        # Order cards.
        #             lColors = ['N'] + referee.Card.COLORS.keys()
        #             lTypes = referee.Card.TYPES
        #             lTypes.reverse()
        #             self.__cards = []
        #             for lColor in lColors:
        #                 for lType in lTypes:
        #                     for lRawCard in self.__player.getHand():
        #                         if lRawCard.type == lType and lRawCard.color == lColor:
        #                             lCard = self.__board.createCard(
        #                                 lRawCard, self.__position % 2 == 0)
        #                             self.__cards.append(lCard)

        # If it was the first deal and the player is AI,
        # notify the referee it is ready.
        if lFirstIdx == 0 and self.__player.isRobot():
            self.__player.ready()


    def onHeapUICb(self, aMessage, aArgs):
        logging.getLogger('duo.ui').debug(
            'gnomish.Player.onHeapUICb %s', aMessage.getPayload())
        # Ensure the move is only made once
        # (we believe here there is only one player on this client)
        if not self.__player.isRobot():
            gobject.idle_add(self.__board.setHeapCard, aMessage.getPayload())


    def onPlayUICb(self, aMessage, aArgs):
        logging.getLogger('duo.ui').debug(
            'gnomish.Player.onPlayUICb %s', aMessage.getPayload())
        lPlayer, lRawCard, lColor, lResult = aMessage.getPayload()
        if lPlayer == self.getName():

            # Get the card image from its raw representation.
            lCard = None
            for lIdx in xrange(len(self.__cards)):
                if self.__cards[lIdx].getRawCard() == lRawCard:
                    lCard = self.__cards[lIdx]
                    break

            # If the player is AI, we need to first show the move.
            if self.__player.isRobot():
                gobject.idle_add(self.__board.playCard, lCard, self)

            # Finalize the current move.
            if lResult == True:
                self.__cards.remove(lCard)
                #gobject.idle_add(self.__board.packCards, self)
            else:
                gobject.idle_add(self.__board.revertCard, lCard, self, lIdx)
                #self.__drawButton.set_state(gtk.STATE_NORMAL)
                #self.__passButton.set_state(gtk.STATE_NORMAL)


    def onErrorUICb(self, aMessage, aArgs):
        logging.getLogger('duo.ui').debug(
            'gnomish.Player.onErrorUICb %s', aMessage.getPayload())
        # TODO:
        # Do not issue a dialog ! Perhaps a message in status bar, or
        # a window for this kind of events and chat.


    def onTurnUICb(self, aMessage, aArgs):
        logging.getLogger('duo.ui').debug(
            'gnomish.Player.onTurnUICb %s', aMessage.getPayload())
        lName = aMessage.getPayload()
        if lName == self.__player.getName() and not self.__player.isRobot():
            self.__drawButton.set_state(gtk.STATE_NORMAL)
            self.__passButton.set_state(gtk.STATE_NORMAL)


    def onPassUICb(self, aMessage, aArgs):
        logging.getLogger('duo.ui').debug(
            'gnomish.Player.onPassUICb %s', aMessage.getPayload())


    def onReverseUICb(self, aMessage, aArgs):
        logging.getLogger('duo.ui').debug('gnomish.Player.onReverseUICb')



class UserInterface(object):
    ROOT_WINDOW = 'main_window'
    PLAYERS_BOX = 'players_vbox'
    ABOUT_DIALOG = 'about_dialog'

    def __init__(self):
        logging.getLogger('duo.ui').debug('gnomish.UserInterface.__init__')

        # Load the Glade design and build the window.
        self.__design = gtk.glade.XML(GLADE_FILE, self.ROOT_WINDOW)
        self.__board = Board(self, self.__design)
        self.__playersBox = self.__design.get_widget(self.PLAYERS_BOX)

        # Plug callbacks.
        self.__design.signal_autoconnect(
            {
            #'on_new_game_activate': (self.onMenuNewGame),
            'on_new_game_activate': (self.test_only),
            'on_draw_activate': (self.onMenuDraw),
            'on_pass_activate': (self.onMenuPass),
            'on_quit_activate': (gtk.main_quit),
            'on_rules_activate': (self.onMenuRules),
            'on_preferences_activate': (self.onMenuPreferences),
            'on_about_activate': (self.onMenuAbout),
            'on_main_window_delete_event': (gtk.main_quit)
            }
        )

        # Game data.
        self.__players = []

        # TODO Catch logs in the program ?
        #  log.createWithSocket('achille', aLevel = log.DEBUG)


    def mainloop(self):
        gtk.main()


    def getBoard(self):
        return self.__board


    def getPlayers(self):
        return self.__players


    def registerPlayerUI(self, aBox):
        self.__playersBox.pack_start(aBox)


    def onMenuNewGame(self, aWidget):
        """Ask the user for the new game parameters.
        """
        lDialog = NewGameDialog()
        if lDialog.run() == gtk.RESPONSE_OK:
            self.startNewGame(lDialog.getGameParams())
        return gtk.FALSE


    def onMenuDraw(self, aWidget):
        pass


    def onMenuPass(self, aWidget):
        pass


    def onMenuRules(self, aWidget):
        pass


    def onMenuPreferences(self, aWidget):
        pass


    def onMenuAbout(self, aWidget):
        lXmlDesign = gtk.glade.XML(GLADE_FILE, self.ABOUT_DIALOG)
        lXmlDesign.get_widget(self.ABOUT_DIALOG).run()
        return gtk.FALSE


    def startNewGame(self, aGameParams):
        logging.getLogger('duo.ui').debug(
            'gnomish.UserInterface.startNewGame')

        lName, lNbAIPlayers, lProtocol, lOptions = aGameParams

        # Draw the current player

        # Put the Deck

        # Initialize points

        # Create players
        self.__players.append(Player(self, lName, lProtocol, lOptions))
        for lIdx in range(lNbAIPlayers):
            self.__players.append(Player(
                self, '', lProtocol, lOptions,
                aPosition = lIdx + 1, aRobot = True))


    def test_only(self, aWidget):
        import time
        self.__playersBox.remove(self.__playersBox.get_children()[0])
        self.__players.append(
            Player(self, 'david', 'local',
                   [ protocols.Option('nb_players', 2, 'Number of players') ]))
        self.__players.append(
            Player(self, '', 'local',
                   [ protocols.Option('nb_players', 2, 'Number of players'),
                     protocols.Option('ai', 'camille', 'AI')],
                   aPosition = 1))
